@@ -1,5 +1,6 @@
 import asyncio
 import builtins
+import time
 import traceback
 from textwrap import dedent
 
@@ -306,6 +307,46 @@ def test_non_str_input():
     )
 
 
+def test_custom_non_str_input():
+    class Runner(MyRunner):
+        def non_str_input(self, value):
+            print(repr(value))
+
+    def callback(event_type, data):
+        if event_type == "input":
+            return 123
+        return default_callback(event_type, data)
+
+    runner = Runner(callback=callback)
+    assert runner.line == ""
+
+    check_simple(
+        "input()",
+        runner=runner,
+        expected_events=[
+            (
+                "output",
+                {
+                    "parts": [
+                        {"text": "", "type": "input_prompt"},
+                    ],
+                },
+            ),
+            (
+                "output",
+                {
+                    "parts": [
+                        {"text": "123\n", "type": "stdout"},
+                        {"text": "\n", "type": "input"},
+                    ]
+                },
+            ),
+        ],
+    )
+
+    assert runner.line == ""
+
+
 def test_single():
     check_simple(
         "1 + 2",
@@ -504,3 +545,45 @@ def test_await():
             },
         ),
     ]
+
+
+def test_async_without_await():
+    global events
+    events = []
+
+    runner = MyRunner(callback=default_callback)
+    result = runner.run_async(
+        dedent(
+            """
+    def foo():
+        print('hi')
+
+    foo()
+            """
+        )
+    )
+    assert events == []
+    asyncio.run(result, debug=True)
+    assert events == [
+        (
+            "output",
+            {
+                "parts": [
+                    {"type": "stdout", "text": "hi\n"},
+                ],
+            },
+        ),
+    ]
+
+
+def test_invalid_sleep():
+    runner = PatchedSleepRunner()
+    for arg in [-1, float('nan'), "", 1+2j, None]:
+        with pytest.raises(Exception):
+            try:
+                time.sleep(arg)
+            except Exception as e:
+                assert isinstance(e, (ValueError, TypeError))
+                with pytest.raises(type(e)):
+                    runner.sleep(arg)
+                raise
